/**
 * License Agreement.
 *
 * JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */ 
package org.richfaces.testng;

import org.ajax4jsf.bean.A4JActionParam;
import org.ajax4jsf.template.Template;
import org.richfaces.SeleniumTestBase;
import org.testng.Assert;
import org.testng.annotations.Test;

public class AjaxActionParameterTest extends SeleniumTestBase {

    /**
     * Tests ajax action parameter component
     * 
     * @param template
     */
    @Test
    public void testAjaxActionParameterComponent(Template template) {
        renderPage(template);
        String parentId = getParentId();
        String parameter = null;

        String paramID = parentId + "_parameter";
        String ajaxButttonID = parentId + "_form1:ajaxSubmit";
        String htmlButttonID = parentId + "_form2:htmlSubmit";

        // test ajax submit
        clickById(ajaxButttonID);
        waitForAjaxCompletion();
        parameter = getTextById(paramID);
        checkParameter(parameter);

        // test html submit
        clickCommandAndWait(htmlButttonID);
        parameter = getTextById(paramID);
        checkParameter(parameter);
    }

    /**
     * Checks action parameter
     * 
     * @param parameter -
     *                Action Parameter value
     */
    private void checkParameter(String parameter) {
        if ("".equals(parameter)) {
            writeStatus("<a4j:actionParameter> failed. ActionListener skipped.", true);
            Assert.fail("<a4j:actionParameter> failed. ActionListener skipped.");
        } else if (parameter.equals(A4JActionParam.ERRORS.NO_PARAM)) {
            writeStatus("<a4j:actionParameter> failed. Parameter has been not sent with ajax request.", true);
            Assert.fail("<a4j:actionParameter> failed. Parameter has been not sent with ajax request.");
        } else if (parameter.equals(A4JActionParam.ERRORS.NOT_ASSIGNED)) {
            writeStatus("<a4j:actionParameter> failed. Parameter value has been not assigned to bean", true);
            Assert.fail("<a4j:actionParameter> failed. Parameter value has been not assigned to bean");
        } else if (parameter.equals(A4JActionParam.ERRORS.INVALID_VALUE)) {
            writeStatus("<a4j:actionParameter> failed. Parameter value is incorrect", true);
            Assert.fail("<a4j:actionParameter> failed. Parameter value is incorrect");
        } else {
            writeStatus("<a4j:actionParameter> passed successfully");
        }
    }

    @Override
    public String getTestUrl() {
        return "pages/actionParam/actionParam.xhtml";
    }

}
